package org.greenstone.server;

import gate.Annotation;
import gate.AnnotationSet;
import gate.Corpus;
import gate.CorpusController;
import gate.Document;
import gate.Factory;
import gate.FeatureMap;
import gate.Gate;
import gate.GateConstants;
import gate.ProcessingResource;
import gate.corpora.RepositioningInfo;
import gate.creole.ANNIEConstants;
import gate.creole.ResourceInstantiationException;
import gate.creole.SerialAnalyserController;
import gate.util.GateException;
import gate.util.Out;
import gate.util.persistence.PersistenceManager;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.apache.log4j.BasicConfigurator;

public class GateScanner 
{
	private SerialAnalyserController annieController;

	public GateScanner()
	{
		try 
		{
			BasicConfigurator.configure();
			Gate.setGateHome(new File("/home/sam/Desktop/Research/Gate5"));
			Gate.setPluginsHome(new File("/home/sam/Desktop/Research/Gate5/plugins"));
			Gate.setUserConfigFile(new File("/home/sam/Desktop/Research/Gate5/gate.xml"));
			Gate.init();
			Gate.getCreoleRegister().registerDirectories(new URL("file:///home/sam/Desktop/Research/Gate5/plugins/ANNIE")); 
			Gate.getUserConfig().put(GateConstants.DOCUMENT_ADD_SPACE_ON_UNPACK_FEATURE_NAME, new Boolean(false));
			
		    Out.prln("Initialising ANNIE...");
		
			annieController = (SerialAnalyserController) PersistenceManager.loadObjectFromFile(new File(new File(Gate.getPluginsHome(), ANNIEConstants.PLUGIN_DIR), ANNIEConstants.DEFAULT_FILE));
		
		    Out.prln("...ANNIE loaded");
		} 
		catch (Exception e) 
		{
			e.printStackTrace();
		}
	}
	
	/** Tell ANNIE's controller about the corpus you want to run on */
	public void setCorpus(Corpus corpus) 
	{
		annieController.setCorpus(corpus);
	} // setCorpus

	/** Run ANNIE */
	public void execute() throws GateException 
	{
	    Out.prln("Running ANNIE...");
	    annieController.execute();
	    Out.prln("...ANNIE complete");
	} // execute()
	
	public ArrayList<Word> classifyText(String originalText)
	{
		ArrayList<Word> words = new ArrayList<Word>();
		try
		{
		    // create a GATE corpus and add a document for each command-line
		    // argument
		    Corpus corpus = (Corpus) Factory.createResource("gate.corpora.CorpusImpl");
		    FeatureMap params = Factory.newFeatureMap();
		    params.put("stringContent", "<html><head></head><body>" + originalText + "</body></html>");
		    params.put("preserveOriginalContent", new Boolean(true));
		    params.put("collectRepositioningInfo", new Boolean(true));
		    Document doc = (Document) Factory.createResource("gate.corpora.DocumentImpl", params);
		    corpus.add(doc);
	
		    // tell the pipeline about the corpus and run it
		    this.setCorpus(corpus);
		    this.execute();	    
		    
		    AnnotationSet annotations = doc.getAnnotations();
		    
		    int index = 0;
		    for(int j = 0; j < annotations.size(); j++)
		    {
		    	Annotation currentAnnotation = annotations.get(j);
		    	
		    	if(currentAnnotation == null)
		    	{
		    		continue;
		    	}
		    	if(currentAnnotation.getType().equals("Token"))
		    	{
		    		Word w = new Word();
		    		w.setIndex(index++);
		    		w.setClassification((String)currentAnnotation.getFeatures().get("category"));
		    		w.setValue((String)currentAnnotation.getFeatures().get("string"));
		    		
		    		words.add(w);
		    	}
		    }
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		
	    return words;
	}
}